/*  Liteshow JavaScript library
 *  (c) 2006-2007 Benjamin Mack <bmack at xnos.org>
 *
 *  Liteshow is freely distributable under the terms of an MIT-style license.
 *  For details, see the Liteshow web site at http://www.xnos.org/
**/

// extending the window object 
Object.extend(window, {
	getInnerWidth: function() {
		// Non-IE
		if (typeof(window.innerWidth) == 'number') return window.innerWidth;
		// IE 6+ in 'standards compliant mode
		if (document.documentElement && document.documentElement.clientWidth) return document.documentElement.clientWidth;
		// IE 4 compatible
		if (document.body && document.body.clientWidth) return document.body.clientWidth;
	},

	getInnerHeight: function() {
		if (typeof(window.innerHeight) == 'number') return window.innerHeight;
		if (document.documentElement && document.documentElement.clientHeight) return document.documentElement.clientHeight;
		if (document.body && document.body.clientHeight) return document.body.clientHeight;
	}
});


// ==== Liteshow class ==== //
var Liteshow = {
	className: 'Liteshow',
	ids: {},
	browser: {},
	pos: 0,
	
	
	// ================ XML regarding functions ============ //
	xml: null,

	// -- loading the XML file
	loadXML: function(xmlfile) { new Ajax.Request(this.options.pathprefix + xmlfile, { method: 'get', onComplete: this.postLoadXML.bind(this) } ); },

	// -- done loading the XML file
	postLoadXML: function(req) {
		this.xml = req.responseXML;
		this.processEvent('xml');
	},

	// returns the information from the XML file of a image in an array with two elements
	getImageInfo: function(pos) {
		if (!this.isValidImagePos(pos)) return [];
		var node = this.xml.getElementsByTagName('image');
		var src   = this.options.pathprefix + this.options.id + '/' + node[pos].getAttribute('src');
		var title = node[pos].getElementsByTagName('title')[0].firstChild.nodeValue;
		return [src, title];
	},

	isLastImage:     function(pos) { return (pos == this.numImages()-1)          ? true : false; },
	isValidImagePos: function(pos) { return (pos < 0 || pos >= this.numImages()) ? false : true; },
	numImages:       function()    { return this.xml.getElementsByTagName('image').length || 0; },

	// ============== functions regarding event framework =========== //
	events: {},
	addEvent: function(key, func) { this.events[key] = func; },
	processEvent: function(key) {
		if (this.events[key]) {
			this[this.events[key]](key);
			this.events[key] = null;
		}
	},

	// instantiates the object with the xml file, loads the XML and then starts the slide show
	init: function(id) {
		var defaultOptions = {
			id: id,
			container: 'liteshow',
			overlay: 'overlay',
			pathprefix: 'galleries/',
			startPos: 0,
			maxHeight: 80, // screen height in percent that the image can fil out max
			pagetitle: null,
			controlContainer: 'liteshowControls',
			cacheContainer: 'liteshowCache',
			cacheNum: 4,
			transitionTime: 2,
			timer: true,
			timerDelay: 5,
			automaticRewind: true
		};
		this.options = Object.extend(defaultOptions, (arguments[1] || {}) );
		this.pos = this.options.startPos;

		// initialize timer
		LiteTimer.setOptions(this.className, this.options.timer, this.options.timerDelay,  this.options.transitionTime);

		// -- DOM manipulation --
		// creating the container, the overlay and the cacheContainer, then show the overlay
		var objBody = document.getElementsByTagName("body")[0];
		this.ids.bodyParams = objBody.style.overflow;
		objBody.style.overflow = 'hidden';
		if (!$(this.options.overlay))   new Insertion.Top(objBody, '<div id="' + this.options.overlay   + '" style="display: none;"></div>');
		if (!$(this.options.container)) new Insertion.Top(objBody, '<div id="' + this.options.container + '"></div>');
		else {
			// reset to default styles when starting a new show
			$(this.options.container).style.display = 'none';
			$(this.options.container).innerHTML = '';
			$(this.options.container).style.top = "0px";
			$(this.options.container).style.left = "0px";
			$(this.options.container).style.width = "100%";
			$(this.options.container).style.height = "100%";
		}
		if (!$(this.options.cacheContainer)) new Insertion.Bottom(this.options.container, '<div id="' + this.options.cacheContainer + '"></div>');
		new Effect.Appear(this.options.overlay, { duration: 0.2, from: 0.0, to: 0.7 });
		this.showLoadingIcon();

		// load XML file
		this.addEvent('xml', 'setupShow');
		this.loadXML(id + '.xml');
	},
	
	setupShow: function() {
		var waitTilCached = this.pos+this.options.cacheNum-1;
		while (waitTilCached >= this.numImages()) waitTilCached--;
		if (this.isImageCached(waitTilCached)) {
			this.firstImage();
		} else {
			this.addEvent(waitTilCached, 'firstImage');
			this.cacheImages(this.pos);
		}

		// adding control mechanisms
		this.setupControls();
		Event.observe(document, 'keypress', this._keyboardControl.bindAsEventListener(this), true);
	},

	// clean up show elements
	endShow: function() {
		this.hideLoadingIcon();
		LiteTimer.stop();
		LiteTimer.disposeSlider();
		new Effect.SlideUp(this.options.container, { delay: 0.2, duration: 0.7 } );
		new Effect.Fade(this.options.overlay, { duration: 0.8, delay: 1, afterFinish: function() { document.getElementsByTagName("body")[0].style.overflow = this.ids.bodyParams; }.bind(this) } );
		Event.stopObserving(document, 'keypress', this._keyboardControl.bindAsEventListener(this), true);
		LiteBookmark.clearHash();
	},


	rewindShow: function() {
		var timer = LiteTimer.state;
		if (timer) LiteTimer.stop();
		if (this._rewindShowHook)
			this._rewindShowHook();
		else {
			this.pos = 0;
			this.viewImage(this.pos);
		}
		if (timer) LiteTimer.start();
	},

	// function that checks if an arrow key is pressed, then slides the images
	_keyboardControl: function(e) {
		var key = (e.which) ? e.which : e.keyCode;
		Event.stop(e);
		if (key == Event.KEY_RIGHT || key == Event.KEY_DOWN) { this.nextImage(); return false; }
		if (key == Event.KEY_LEFT  || key == Event.KEY_UP)   { this.previousImage(); return false; }
		LiteTimer.stop();
		return true;
	},


	// ============== Image Caching Functions =========== //

	isImageCached: function(pos) { return (this.imageCached(pos) && this.imageCached(pos).complete) ? true : false; },

	// caches the next X and the previous X image(s)
	cacheImages: function(pos) {
		if (!this.isValidImagePos(pos)) return false;
		var startPreloadPos = (pos-this.options.cacheNum > 0) ? pos-this.options.cacheNum : 0;
		var stopPreloadPos  = pos+this.options.cacheNum;
		for (var i = startPreloadPos; i < stopPreloadPos; i++) {
			this._cacheImage(i);
			if (this.isLastImage(i)) break;
		}
	},

	// internal function to create a cache image and set an event when loaded
	_cacheImage: function(pos) {
		var id = 'ls_cache_img_' + pos;
		// no need to load an already cached image
		if ($(id)) return;
		var info = this.getImageInfo(pos);
		new Insertion.Bottom(this.options.cacheContainer, '<img src="' + info[0] + '" id="' + id + '"/>');
		$(id).onload = this.processEvent.bind(this, pos);
	},


	// == Image Loading Functions == // 

	// takes the preloaded image and uses it as a real image
	_createImage: function(pos) {
		if (!this.isValidImagePos(pos)) return false;
		if (this.image(pos)) return true;
		if (!this.isImageCached(pos)) this._cacheImage(pos);
		var info = this.getImageInfo(pos);
		
		var objImg   = Builder.node('img', {id:'ls_img_'   + pos, src: info[0], alt: info[1]});
		var objTitle = Builder.node('h4',  {id:'ls_title_' + pos});

		objTitle.innerHTML = '<span>' + info[1] + '</span>';
		var objItem  = Builder.node('div', {id:'ls_' + pos, style: 'display: none'}, [objImg, objTitle]);
		$(this.options.container).appendChild(objItem);
	},


	// the next three fnctions are general functions that take care 
	// of the loading images. Usually this is called from
	// the "firstImage" function of the implementing viewers
	_firstImage: function() {
		Element.show(this.options.container);
		this.__addDefaultImageActions(this.pos);
		LiteBookmark.setHash(this, this.pos);
		if (LiteTimer.state) LiteTimer.start();
		this.hideLoadingIcon();
	},

	_previousImage: function() {
		this.pos = this.pos-1;

		this.__addDefaultImageActions(this.pos);
		LiteBookmark.setHash(this, this.pos);
	},

	_nextImage: function() {
		this.pos = this.pos+1;
		if (this.isLastImage(this.pos)) {
			if (this.options.automaticRewind)
				LiteTimer.rewindShow();
			else
				LiteTimer.stop();
		}
		this.__addDefaultImageActions(this.pos);
		LiteBookmark.setHash(this, this.pos);
	},
	
	_viewImage: function(pos) {
		if (!pos) var pos = this.pos;
		this.__addDefaultImageActions(pos);

		LiteBookmark.setHash(this, this.pos);
	},

	__addDefaultImageActions: function(pos) {
		this.cacheImages(pos);
		if (!this.image(pos))   this._createImage(pos);
		if (!this.image(pos-1)) this._createImage(pos-1);
		if (!this.image(pos+1)) this._createImage(pos+1);

		this.image(pos).onclick = this.nextImage.bind(this);
		if (this.image(pos+1)) this.image(pos+1).onclick = this.nextImage.bind(this);
		if (this.image(pos-1)) this.image(pos-1).onclick = this.previousImage.bind(this);
	},




	// == Peripheral Functions == //
	setupControls: function() {
		if ($(this.options.controlContainer)) return;

		new Insertion.Bottom(this.options.container, '<div id="' + this.options.controlContainer + '"></div>');
		var links = Array();
		links.push('<a href="javascript:void(0);" onclick="Liteshow.' + this.className + '.rewindShow();">First</a>');
		links.push('<a href="javascript:void(0);" onclick="Liteshow.' + this.className + '.previousImage();">Previous</a>');
		links.push('<a href="javascript:void(0);" onclick="Liteshow.' + this.className + '.nextImage();">Next</a>');
		links.push('<a href="javascript:void(0);" onclick="Liteshow.' + this.className + '.browserInit();">Browse</a>');
		links.push('<a href="javascript:void(0);" onclick="Liteshow.' + this.className + '.endShow();">Close</a>');

		for (var i = 0; i < links.length; i++)
			new Insertion.Bottom(this.options.controlContainer, links[i]);

		if (LiteTimer.displaySlider) LiteTimer.initSlider();
	},

	
	showLoadingIcon: function() {
		if (!$('loading')) new Insertion.Top(document.getElementsByTagName('body')[0], '<img id="loading" src="g/loading.gif" style="display: none;"/>');

		if (!Element.visible('loading')) {
			$('loading').style.top = Math.round((window.getInnerHeight()*0.5)-40) + 'px';
			$('loading').style.left = Math.round((window.getInnerWidth()*0.5)-40) + 'px';
			new Effect.Appear('loading', { duration: 0.2, queue: 'loadingicon'} );
		}
	},
	hideLoadingIcon: function() { new Effect.Fade('loading', { duration: 0.2, queue: 'loadingicon' }); },

	// -- default title functions, can be overridden by any view --
	hideTitle: function(pos) { new Effect.Fade(this.title(pos).firstChild,   { duration: 0.5 } ); },
	showTitle: function(pos) { new Effect.Appear(this.title(pos).firstChild, { duration: 0.5, to: 0.6 } ); },


	// == Liteshow Browser == //
	browserInit: function() {
		this.browser.id = 'litebrowser';
		this.browser.images = 0;
		this.browser.tnSize = 250;
		this.browser.tnPadding = 20;
		this.browser.wasTimerOn = LiteTimer.state;
		LiteTimer.stop();
		
		// thumbnail size
		if (!$(this.browser.id)) new Insertion.After(this.options.container, '<div id="' + this.browser.id + '" style="display: none;"></div>');
		else $(this.browser.id).innerHTML = '';

		$(this.browser.id).style.top  = 0;
		$(this.browser.id).style.left = 0;
		$(this.browser.id).style.width  = window.getInnerWidth() + 'px';
		$(this.browser.id).style.height = window.getInnerHeight() - Element.getDimensions(this.options.controlContainer).height + 'px';
		$(this.browser.id).style.display = 'block';
		$(this.browser.id).style.opacity = 1;
		this.showLoadingIcon();
		
		$(this.options.container).style.zIndex = 5;

		// calculate how many images we can bring on the screen
		var numHorz = -1;
		var maxHorzSize = window.getInnerWidth();
		var tmp = maxHorzSize;
		while (tmp > numHorz) {
			numHorz += 1;
			tmp -= this.browser.tnSize+(this.browser.tnPadding*2);
		}
		var horzSize = this.browser.tnSize*numHorz + this.browser.tnPadding*numHorz*2;

		// creating the table datas
		var tbldata = "";
		for (var td = 0; td < this.numImages(); td++) {
			if (td % numHorz == 0) tbldata += '<tr>';
			tbldata += '<td id="browse_td_' + td + '"></td>';
			if (td % numHorz == numHorz-1) tbldata += '</tr>';
		}
		if (td % numHorz != numHorz-1) tbldata += '</tr>';
		$(this.browser.id).innerHTML = '<table cellspacing="0" cellpadding="' + this.browser.tnPadding + '">' + tbldata + '</table>';

		// putting the images in the tds
		for (var pos = 0; pos < this.numImages(); pos++) {
			if (!this.isImageCached(pos)) this._cacheImage(pos);
			this._preloadBrowserImage(pos);
		}
	},


	_preloadBrowserImage: function(pos) {
		var info = this.getImageInfo(pos);
		var browseImg  = Builder.node('img', { id: 'ls_browse_img_' + pos, style: 'display: none;', src: info[0], alt: info[1] });
		$('browse_td_' + pos).appendChild(browseImg);
		var reqId = 'browse_' + pos;
		if (!browseImg.complete) {
			this.addEvent(reqId, '_renderBrowserImage');
			browseImg.onload = this.processEvent.bind(this, reqId);
		} else {
			this._renderBrowserImage(reqId);
		}
	},

	_renderBrowserImage: function(pos) {
		pos = pos.split('_')[1];
		var dims = Element.getDimensions(this.imageCached(pos));
		if (dims.width > dims.height) this.imageBrowser(pos).style.width  = this.browser.tnSize + 'px';
		else                          this.imageBrowser(pos).style.height = this.browser.tnSize + 'px';

		Event.observe(this.imageBrowser(pos), 'click', this.selectBrowserImage.bind(this, pos), false);
		this.browser.images += 1;
		
		var finishUp = function(pos) {
			new Insertion.After(this.imageBrowser(pos), '<span>' + this.imageBrowser(pos).alt + '</span>');
			this._checkBrowserImages();
		};
		new Effect.Appear(this.imageBrowser(pos), { duration: 0.5, queue: { position: 'end', scope: 'browserimages'}, afterFinish: finishUp.bind(this, pos) } );
	},
	
	_checkBrowserImages: function() {
		if (this.browser.images < this.numImages()) return;
		this.hideLoadingIcon();
	},
	
	selectBrowserImage: function(pos) {
		Element.hide(this.item(this.pos));
		pos = parseInt(pos);
		this.hideLoadingIcon();

		// hook after selecting an image from the browser
		if (this._selectBrowserImageHook)
			this._selectBrowserImageHook(pos);

		if (!this.image(pos)) this._createImage(pos);
		Position.clone(this.imageBrowser(pos), this.item(pos), { setWidth: false, setHeight: false } );

		this.pos = pos;
		$(this.options.container).style.zIndex = 20;
		this.item(pos).style.zIndex = 50;
		this.image(pos).style.width  = this.imageBrowser(pos).style.width;
		this.image(pos).style.height = this.imageBrowser(pos).style.height;

		// IE width bug
		if(/MSIE/.test(navigator.userAgent))
			this.item(pos).style.width = this.imageBrowser(pos).style.width;

		Element.show(this.item(pos));
		Element.hide(this.browser.id);
		if (this.browser.wasTimerOn) LiteTimer.start();
		this.viewImage(pos);
	},
	
	// == helper functions for accessing the images, titles and items   == //
	item:			function(pos) { return $('ls_' + pos); },
	title:          function(pos) { return $('ls_title_' + pos); },
	image:			function(pos) { return $('ls_img_' + pos); },
	imageCached: 	function(pos) { return $('ls_cache_img_' + pos); },
	imageBrowser: 	function(pos) { return $('ls_browse_img_' + pos); },
	setMessage:     function(txt) {
		if (!$('liteMessage')) new Insertion.Bottom(this.options.controlContainer, '<span id="liteMessage"></span>');
		$('liteMessage').innerHTML = txt;
		if (this.ids.message) new Effect.Appear('liteMessage', { duration: 0.3 } );
		this.ids.message = new Effect.Fade('liteMessage', { duration: 0.3, delay: 5 } );
	},

	// get maximum dimensions with considering maxHeight / maxWidth option
	getMaxDimensions: function(pos) {
		var dims = Element.getDimensions(this.imageCached(pos));
		while (dims.width > window.getInnerWidth() || dims.height > window.getInnerHeight()*this.options.maxHeight/100) {
			dims.width = dims.width*0.95;
			dims.height = dims.height*0.95;
		}
		return { width: dims.width, height: dims.height };
	},

	
	// == stub functions - these need to be implemented by the extension classes == //
	firstImage:    function() { this._firstImage();    },
	previousImage: function() { this._previousImage(); },
	nextImage:     function() { this._nextImage();     },
	viewImage:     function(pos) {
		if (!pos) var pos = this.pos;
		this._viewImage(pos);
		var dims    = Element.getDimensions(this.imageCached(pos));
		var dimsMax = this.getMaxDimensions(pos);
		var sDims   = Element.getDimensions(this.image(pos));
		var scaleTo = 100;
		var scaleFrom = sDims.width / dims.width * 100;
		if (dims.width  > dimsMax.width)  scaleTo = dimsMax.width  / dims.width * 100;
		if (dims.height > dimsMax.height) scaleTo = dimsMax.height / dims.height * 100;

		var x = Math.floor((window.getInnerWidth() - dimsMax.width)/2);
		var y = Math.floor((window.getInnerHeight() - dimsMax.height)/2 - 20);

		this.image(pos).onclick = this.nextImage.bind(this);
		new Effect.Move(this.item(pos), { delay: 0.3, duration: 0.5, x: x, y: y, mode: 'absolute', afterFinish: function () { this.showTitle(pos); this._createImage(pos+2); }.bind(this) } );
		new Effect.Scale(this.image(pos), scaleTo, { delay: 0.3, duration: 0.5, scaleFrom: scaleFrom, mode: 'absolute', scaleFromCenter: true, scaleMode: { originalHeight: dims.height, originalWidth: dims.width }} );
	}
};


Liteshow.Fader = Class.create();
Object.extend(Object.extend(Liteshow.Fader, Liteshow), {
	className: 'Fader',

	firstImage: function() {
		this._firstImage();
		this.appear();
	},

	previousImage: function() {
		if (!this.isValidImagePos(this.pos-1)) return;
		this._previousImage();

		this.hideVisibleImage(this.pos+1);
		this.hideVisibleImage(this.pos+2);
		this.appear();
	},

	nextImage: function() {
		if (!this.isValidImagePos(this.pos+1)) return;
		this._nextImage();

		this.hideVisibleImage(this.pos-1);
		this.hideVisibleImage(this.pos-2);
		this.appear();
	},
	
	hideVisibleImage: function(pos) {
		if (!this.item(pos)) return false;
		if (Element.visible(this.item(pos)))
			new Effect.Fade(this.item(pos), { queue: { position: 'front', scope: 'image' }, transition: Effect.Transitions.linear, duration: 0.5 } );
	},
	
	appear: function() {
		this.adjustItem(this.pos);
		new Effect.Appear(this.item(this.pos), { queue: { position: 'end', scope: 'image' }, duration: 0.5 } );
		this.showTitle(this.pos);
	},

	adjustItem: function(pos) {
		var dims = this.getMaxDimensions(pos);
		this.item(pos).style.left = Math.floor((window.getInnerWidth()-dims.width)/2) + 'px';
		this.item(pos).style.top  = Math.floor((window.getInnerHeight()-dims.height)/2)-20 + 'px';
		this.item(pos).style.width   = dims.width + 'px';
		this.image(pos).style.width  = dims.width + 'px';
		this.image(pos).style.height = dims.height + 'px';
	},
	
	_rewindShowHook: function() {
		this.hideVisibleImage(this.pos);
		this.pos = 1;
		this._previousImage();
		this.appear();
	}
});

// == Class for view that slides images from the right to the left == 
Liteshow.HorizontalSlide = Class.create();
Object.extend(Object.extend(Liteshow.HorizontalSlide, Liteshow), {
	className: 'HorizontalSlide',

	// implementing default methods
	firstImage: function() {
		this._firstImage();
		
		var dimsMax = this.getMaxDimensions(this.pos);
		this.item(this.pos).style.left = 0 - (dimsMax.width*2);
		this.item(this.pos).style.top = Math.floor(window.getInnerHeight()/2 - dimsMax.height/4);
		this.moveIn(this.pos);
		this.prepareNext(this.pos+1);	
		if (this.pos > 0) this.preparePrevious(this.pos-1);
	},

	previousImage: function() {
		if (!this.isValidImagePos(this.pos-1)) return;
		this._previousImage();

		this.moveOutToRight(this.pos+1);
		this.moveIn(this.pos);
		this.preparePrevious(this.pos-1);
	},

	nextImage: function() {
		if (!this.isValidImagePos(this.pos+1)) return;
		this._nextImage();

		this.moveOutToLeft(this.pos-1);
		this.moveIn(this.pos);
		this.prepareNext(this.pos+1);
	},



	// moves image from the center to the left border of the window
	moveOutToLeft: function(pos) {
		if (!this.item(pos)) return;
		var dims = this.getMaxDimensions(pos);
		var x = Math.floor(0 - dims.width/4);
		var y = Math.floor(window.getInnerHeight()/2 - dims.height/4);

		this.image(pos).onclick = this.previousImage.bind(this);
		new Effect.Move(this.item(pos), { duration: this.options.transitionTime, x: x, y: y, mode: 'absolute', afterFinish: this.hideTitle.bind(this, pos) } );
		new Effect.Scale(this.image(pos), 50, { duration: this.options.transitionTime, scaleFromCenter: true } );
		
		this.unpreparePrevious(pos-1);
	},

	// moves image from the center to the right border of the window
	moveOutToRight: function(pos) {
		if (!this.item(pos)) return;
		var dims = this.getMaxDimensions(pos);
		var x = Math.floor(window.getInnerWidth() - dims.width/4);
		var y = Math.floor(window.getInnerHeight()/2 - dims.height/4);

		this.image(pos).onclick = this.nextImage.bind(this);
		new Effect.Move(this.item(pos), { duration: this.options.transitionTime, x: x, y: y, mode: 'absolute', afterFinish: this.hideTitle.bind(this, pos) });
		new Effect.Scale(this.image(pos), 50, { duration: this.options.transitionTime, scaleFromCenter: true } );

		this.unprepareNext(pos+1);
	},

	// moves image to the center
	moveIn: function(pos) {
		if (!this.item(pos)) return;
		var dims = Element.getDimensions(this.imageCached(pos));
		var dimsMax = this.getMaxDimensions(pos);
		var scaleTo = 100;
		if (dims.width  > dimsMax.width)  scaleTo = dimsMax.width  / dims.width * 100;
		if (dims.height > dimsMax.height) scaleTo = dimsMax.height / dims.height * 100;

		var scaleFrom = Math.floor(scaleTo / 2);
		var x = Math.floor((window.getInnerWidth() - dimsMax.width)/2);
		var y = Math.floor((window.getInnerHeight() - dimsMax.height)/2 - 20);

		if (!Element.visible(this.item(pos))) {
			this.item(pos).style.left = window.getInnerWidth();
			this.item(pos).style.top  = Math.floor(window.getInnerHeight()/2 - dimsMax.height/4);
			this.item(pos).style.display = 'block';
		}
		this.image(pos).onclick = this.nextImage.bind(this);
		
		new Effect.Move(this.item(pos), { duration: this.options.transitionTime, x: x, y: y, mode: 'absolute', afterFinish: function () { this.showTitle(pos); this._createImage(pos+2); }.bind(this) } );
		new Effect.Scale(this.image(pos), scaleTo, { duration: this.options.transitionTime, scaleFrom: scaleFrom, mode: 'absolute', scaleFromCenter: true, scaleMode: { originalHeight: dims.height, originalWidth: dims.width }} );
	},


	// slides image half into the window from the right
	prepareNext: function(pos) {
		if (!this.item(pos)) this._createImage(pos);
		if (!this.item(pos)) return false;

		var dimsMax = this.getMaxDimensions(pos);
		var x = Math.floor(window.getInnerWidth() - dimsMax.width*1/8);
		var y = Math.floor(window.getInnerHeight()/2 - dimsMax.height/4);

		this.image(pos).onclick = this.nextImage.bind(this);
		this.image(pos).style.width  = Math.floor(dimsMax.width/2)  + 'px';
		this.image(pos).style.height = Math.floor(dimsMax.height/2) + 'px';

		this.item(pos).style.left = window.getInnerWidth()*1.2;
		this.item(pos).style.top  = y;
		this.item(pos).style.display = 'block';

		new Effect.Move(this.item(pos), { queue: 'slide', duration: this.options.transitionTime, x: x, y: y, mode: 'absolute'} );
	},

	// slides image half into the window from the left
	preparePrevious: function(pos) {
		if (!this.item(pos)) this._createImage(pos);
		if (!this.item(pos)) return false;

		var dimsMax = this.getMaxDimensions(pos);
		var x = Math.floor(0 - dimsMax.width*3/8);
		var y = Math.floor(window.getInnerHeight()/2 - dimsMax.height/4);

		this.image(pos).onclick = this.previousImage.bind(this);
		this.image(pos).style.width  = Math.floor(dimsMax.width/2)  + 'px';
		this.image(pos).style.height = Math.floor(dimsMax.height/2) + 'px';

		this.item(pos).style.left = '-' + (dimsMax.width*2);
		this.item(pos).style.top  = y;
		this.item(pos).style.display = 'block';

		new Effect.Move(this.item(pos), { queue: 'slide', duration: this.options.transitionTime, x: x, y: y, mode: 'absolute'} );
	},
	
	// slides item out to the right
	unprepareNext: function(pos) {
		if (!this.item(pos)) return;
		var dims = Element.getDimensions(this.imageCached(pos));
		var x = Math.floor(window.getInnerWidth() + dims.width*1.2);
		var y = this.item(pos).style.top;
		new Effect.Move(this.item(pos), { queue: 'slide', duration: this.options.transitionTime, x: x, y: y, mode: 'absolute'});
	},
	
	// slides item out to the left
	unpreparePrevious: function(pos) {
		if (!this.item(pos)) return;
		var dims = Element.getDimensions(this.imageCached(pos));
		var x = -(dims.width*1.2);
		var y = this.item(pos).style.top;
		new Effect.Move(this.item(pos), { queue: 'slide', duration: this.options.transitionTime, x: x, y: y, mode: 'absolute'});
	},
	
	// adding hook functions
	_selectBrowserImageHook: function(pos) {
		// hide every visible image
		for (var i = 0; i < this.numImages(); i++) {
			if (this.item(i) && Element.visible(this.item(i)))
				Element.hide(this.item(i));
		}
		this.prepareNext(pos+1);
		this.preparePrevious(pos-1);
	},

	_rewindShowHook: function() {
		if (this.pos == 0) return;
		if (this.pos == 1) return this.previousImage();
		this.unprepareNext(this.pos+1);
		this.unprepareNext(this.pos);
		if (this.pos != 2) this.unprepareNext(this.pos-1);

		this.pos = 0;
		this.__addDefaultImageActions(this.pos);
		var dimsMax = this.getMaxDimensions(this.pos);
		var x = Math.floor(0 - dimsMax.width*2);
		var y = Math.floor(window.getInnerHeight()/2 - dimsMax.height/4);

		this.image(this.pos).style.width  = Math.floor(dimsMax.width/2)  + 'px';
		this.image(this.pos).style.height = Math.floor(dimsMax.height/2) + 'px';
		this.item(this.pos).style.left = x;
		this.item(this.pos).style.top  = y;
		this.item(this.pos).style.display = 'block';
		this.viewImage(this.pos);
		this.prepareNext(this.pos+1);
	}

});


Liteshow.SlideDown = Class.create();
Object.extend(Object.extend(Liteshow.SlideDown, Liteshow), {
	className: 'SlideDown',
	effectInProgress: null,

	firstImage: function() {
		this._firstImage();
		this.appear();
	},

	previousImage: function() {
		if (!this.isValidImagePos(this.pos-1)) return;
		this._previousImage();

		this.hideVisibleImage(this.pos+1);
		this.hideVisibleImage(this.pos+2);
		this.appear();
	},

	nextImage: function() {
		if (!this.isValidImagePos(this.pos+1)) return;
		this._nextImage();

		this.hideVisibleImage(this.pos-1);
		this.hideVisibleImage(this.pos-2);
		this.appear();
	},
	
	hideVisibleImage: function(pos) {
		if (!this.item(pos)) return false;
		if (Element.visible(this.item(pos))) {
			new Effect.SlideUp(this.item(pos), { queue: { position: 'front', scope: 'image' }, transition: Effect.Transitions.linear, duration: this.options.transitionTime, delay: 0.3 } );
		}
	},
	
	appear: function() {
		this.adjustItem(this.pos);
		if (this.effectInProgress) {
			this.effectInProgress.cancel();
		}
		this.effectInProgress = new Effect.SlideDown(this.item(this.pos), { queue: { position: 'end', scope: 'image' }, duration: this.options.transitionTime, delay: 0.3, afterFinish: this.showTitle.bind(this, this.pos) } );
	},

	adjustItem: function(pos) {
		var dims = this.getMaxDimensions(pos);
		this.item(pos).style.left = Math.floor((window.getInnerWidth()-dims.width)/2) + 'px';
		this.item(pos).style.top  = Math.floor((window.getInnerHeight()-dims.height)/2)-20 + 'px';
		this.item(pos).style.width   = dims.width + 'px';
		this.image(pos).style.width  = dims.width + 'px';
		this.image(pos).style.height = dims.height + 'px';
	},
	
	_rewindShowHook: function() {
		this.hideVisibleImage(this.pos);
		this.pos = 1;
		this._previousImage();
		this.appear();
	},
	
	// overriding default methods
	hideTitle: function(pos) { Element.hide(this.title(pos).firstChild); },
	showTitle: function(pos) { Element.setOpacity(this.title(pos).firstChild, '0.6'); Element.show(this.title(pos).firstChild); }
});


// ==== Timer class ==== //
LiteTimer = {
	showName: 'Liteshow', // name of the view currently running
	state: null,
	delay: null,
	transitionTime: null,

	slider: null,
	displaySlider: true,
	sliderHandle: 'liteTimerHandle',
	sliderTrack: 'liteTimer',

	id: null,
	trackObj: null,
	clickOff: null,

	setOptions: function(showName, state, delay, transitionTime) {
		this.showName = showName;
		this.state = state;
		this.delay = delay;
		this.transitionTime = transitionTime;
	},
	
	start: function(delay) {
		if (!delay) var delay = this.delay;
		else this.delay = parseInt(delay);
		this.state = true;
		this.sliderMessage(delay);
		this.reset();
	},

	reset: function() {
		if (!this.state) return;
		window.clearTimeout(this.id);
		this.id = window.setTimeout('Liteshow.' + this.showName + '.nextImage(); LiteTimer.reset();', (this.delay+this.transitionTime) * 1000);
		this.clickOff = this.stop.bind(this);
		window.onmousedown = this.clickOff;	
	},

	stop: function() {
		Liteshow[this.showName].setMessage('Timer off');
		this.state = false;
		window.clearTimeout(this.id);
		if (this.clickOff) {
			window.onmousedown = '';
			this.clickOff = null;
		}
	},
	
	initSlider: function() {
		if (!this.trackObj)
			new Insertion.Top(Liteshow[this.showName].options.controlContainer, '<span id="' + this.sliderTrack + '"><span id="' + this.sliderHandle + '"></span></span>');
		else
			$(Liteshow[this.showName].options.controlContainer).appendChild(this.trackObj);
		
		if (!this.slider) {
			this.slider = new Control.Slider(this.sliderHandle, this.sliderTrack, { range: $R(this.transitionTime, 30), sliderValue: this.delay } );
			this.slider.options.onChange = function(value) { value = value.toFixed(); this.start(value); }.bind(this);
			this.slider.options.onSlide = function(value) { value = value.toFixed(); this.sliderMessage(value); }.bind(this);
		}
	},
	
	sliderMessage: function(delay) {
		var message = 'Timer set to ' + delay + ' seconds.';
		Liteshow[this.showName].setMessage(message);
		$(this.sliderHandle).setAttribute('title', message);
	},

	disposeSlider: function() {
		this.trackObj = $(this.sliderTrack);
	},
	
	rewindShow: function() {
		this.id = window.setTimeout('Liteshow.' + this.showName + '.rewindShow(); LiteTimer.reset();', this.delay * 2 * 1000);
	}
};

// ==== Bookmarker class ==== //
// 0: ShowName, 1: GalleryID, 2: Position

LiteBookmark = {
	recoverShow: function() {
		var hash = window.location.hash;
		if (hash) {
			hash = hash.substring(1, hash.length);
			var details = hash.split('_');
			if (details.length != 3) return;
			Liteshow[details[0]].init(details[1], { startPos: parseInt(details[2]) });
		}
	},
	setHash:   function(show, pos) { window.location.hash = '#' + show.className + '_' + show.options.id + '_' + pos; },
	clearHash: function() { window.location.hash = '#'; }
};

Event.observe(window, 'load', LiteBookmark.recoverShow, false);